@@ -21,7 +21,6 @@ from .core import TemplateCollection, TemplateError, TemplateInfo
 HIDDEN_VAR = re.compile(r"^_.*")
 
 DO_NOT_INHERIT = [
-    "layer",
     HIDDEN_VAR,
 ]
 
@@ -89,7 +88,8 @@ def do_config(template_collection: TemplateCollection, args):
         if n > 1:
             layer_name += f"-{n}"
 
-        layer_mounts.extend(l for l in tmp.default_layer_mounts if l not in layer_mounts)
+        layer_mounts.extend(l for l in tmp.default_layer_mounts
+                            if l not in layer_mounts)
         '''
         # Prompt user
         layer_name_prompt = input(f"Layer name?  [{layer_name}] ")
@@ -162,7 +162,8 @@ def is_template(value):
         return False
 
 
-def nested_expand_missing(data: dict, context: dict, template: TemplateInfo, inherited_vars: dict, env: Environment, path=()):
+def nested_expand_missing(data: dict, context: dict, template: TemplateInfo,
+                          inherited_vars: dict, env: Environment, path=()):
     if isinstance(data, dict):
         output = {}
         for (key, value) in data.items():
@@ -174,15 +175,21 @@ def nested_expand_missing(data: dict, context: dict, template: TemplateInfo, inh
             output[key] = value
         return output
     elif isinstance(data, list):
-        return [nested_expand_missing(d, context, template, inherited_vars, env, path + (i,)) for (i, d) in enumerate(data)]
+        return [nested_expand_missing(d, context, template, inherited_vars, env, path + (i,))
+                for (i, d) in enumerate(data)]
     elif is_template(data):
         return render_variable(env, data, context)
     return data
 
 
-def generate_layer(template: TemplateInfo, layer: dict, tmp_path: Path, repo_path: str, inherited_vars: dict = None, verbose: bool = False):
+def generate_layer(template: TemplateInfo, layer: dict, crispy_var: dict,
+                   tmp_path: Path, repo_path: str, inherited_vars: dict = None,
+                   verbose: bool = False):
     data = layer["cookiecutter"]
-    context = {"cookiecutter": data}
+    context = {
+        "cookiecutter": data,
+        "crispycookie": crispy_var
+    }
     env = StrictEnvironment(context=context)
 
     if verbose:
@@ -224,7 +231,7 @@ def generate_layer(template: TemplateInfo, layer: dict, tmp_path: Path, repo_pat
     out_dir.mkdir(parents=True)
     template_path = str(template.path)
     context["cookiecutter"]["_template"] = f"{repo_path}/{template.path.name}"
-    context["cookiecutter"]["_layer"] = layer['layer_name']
+    context["crispycookie"]["layer_name"] = layer['layer_name']
     # Run cookiecutter in a temporary directory
     project_dir = generate_files(template_path, context, output_dir=str(out_dir))
     #out_projects = [i for i in out_dir.iterdir() if i.is_dir()]
@@ -242,11 +249,16 @@ def generate_layer(template: TemplateInfo, layer: dict, tmp_path: Path, repo_pat
         if block:
             print(f"Preventing explicit retention of {key} because {block}")
             data.pop(key)
-        elif key.startswith("_"):
-            pass
         else:
             print(f"Retaining {key} because it was explicitly set")
 
+    # Remove vars that we added
+    context["cookiecutter"].pop("_template")
+
+    # To address backwards compatibility with my templates
+    if "_template_version" in context["cookiecutter"]:
+        del context["cookiecutter"]["_template_version"]
+
     if inherited_vars is not None:
         inherited_vars.update(dict_without_keys(data, *DO_NOT_INHERIT))
 
@@ -266,7 +278,8 @@ def do_build(template_collection: TemplateCollection, args):
     else:
         config_file = output / ".crispycookie.json"
         if not config_file.is_file():
-            print(f"Missing {config_file} file.  Refusing to rebuild {output.name}", file=sys.stderr)
+            print(f"Missing {config_file} file.  "
+                  "Refusing to rebuild {output.name}", file=sys.stderr)
             return 1
         print(f"Regenerating a project {output.name} from existing {config_file.name}")
         # This seems silly, but to keep with the existing convention
@@ -274,50 +287,78 @@ def do_build(template_collection: TemplateCollection, args):
         with open(config_file) as f:
             config = json.load(f)
 
+    build_project(template_collection, output_folder, config,
+                  project_parent=output,
+                  verbose=verbose, overwrite=args.overwrite)
+
+
+def build_project(template_collection: TemplateCollection, project_dir: Path,
+                  config: dict, project_parent: Path = None,
+                  verbose: bool = False, overwrite: bool = False):
+    # XXX: Figure out a better way to handle project_dir / project_parent
+
     layers = config["layers"]
     inheritance_store = {}
 
+    if "layer_mounts" in config:
+        mount_points = config["layer_mounts"]
+    else:
+        print("No layers have been defined.  To enable this, add "
+              "'layer_mounts' to the configuration file.")
+        mount_points = []
+
+    # These are available as {{ crispycookie.layer_mounts }};
+    # For backwards compatibility with purse cookiecutter, use:
+    #   {{ crispycookie.layer_mounts | default([]) }}
+    crispycookie_var = {
+        "layer_mounts": mount_points,
+    }
+
     with TemporaryDirectory() as tmp_dir:
         tmpdir_path = Path(tmp_dir)
         layer_dirs = []
         for layer in layers:
-            print(f"EXECUTING cookiecutter {layer['name']} template for layer {layer['layer_name']}")
+            print(f"EXECUTING cookiecutter {layer['name']} template for layer "
+                  f"{layer['layer_name']}")
             template = template_collection.get_template(layer["name"])
-            layer_dir = generate_layer(template, layer, tmpdir_path, args.repo,
-                                       inherited_vars=inheritance_store, verbose=verbose)
+            layer_dir = generate_layer(template, layer, crispycookie_var,
+                                       tmpdir_path, template_collection.repo,
+                                       inherited_vars=inheritance_store,
+                                       verbose=verbose)
             layer_dirs.append(layer_dir)
             print("")
 
         top_level_names = set(ld.name for ld in layer_dirs)
         if len(top_level_names) > 1:
-            raise ValueError(f"Found inconsistent top-level names of generated folders... {top_level_names}")
+            raise ValueError(f"Found inconsistent top-level names of generated "
+                             f"folders... {top_level_names}")
         top_level = top_level_names.pop()
 
         stage_folder = tmpdir_path / top_level
 
-        if output_folder is None:
-            output_folder = output / top_level
+        if project_dir is None:
+            project_dir = project_parent / top_level
 
-        if output_folder.is_dir():
-            if args.overwrite:
-                folder_name = output_folder.absolute().name if output_folder.name == "" else output_folder
+        if project_dir.is_dir():
+            if overwrite:
+                if project_dir.name == "":
+                    folder_name = project_dir.absolute().name
+                else:
+                    folder_name = project_dir
                 sys.stderr.write(f"Overwriting output directory {folder_name}, as requested.\n")
             else:
                 sys.stderr.write(" *******************  ABORT  *******************\n\n")
-                sys.stderr.write(f"Output directory {output_folder.absolute()} already exists.  "
+                sys.stderr.write(f"Output directory {project_dir.absolute()} already exists.  "
                                  "Refusing to overwrite.\n")
                 sys.stderr.write("\n")
                 sys.exit(1)
 
-        mount_points = config.get("layer_mounts", [])
         if mount_points:
             print(f"Applying project mount points:  {mount_points}")
             for i, layer_dir in enumerate(layer_dirs):
                 layer_info = layers[i]
                 layer_name = layer_info["layer_name"]
                 move_to_layers(layer_dir, layer_name, mount_points)
-        else:
-            print("No layers have been defined.  To enable this, add 'layer_mounts' to the configuration file.")
 
         print("Combining cookiecutter layers")
         # Combine all cookiecutter outputs into a single location
@@ -327,22 +368,97 @@ def do_build(template_collection: TemplateCollection, args):
             layer_name = layer_info["layer_name"]
             _copy_tree(layer_dir, stage_folder, layer_info=layer_name)
 
-        print(f"Copying generated files to {output_folder}")
-        _copy_tree(stage_folder, output_folder)
+        print(f"Copying generated files to {project_dir}")
+        _copy_tree(stage_folder, project_dir)
 
     for layer in layers:
         for clean_var in ["_extensions"]:
             if clean_var in layer["cookiecutter"]:
                 del layer["cookiecutter"][clean_var]
 
+    config["source"] = {
+        "repo": template_collection.repo,
+        "rev": template_collection.rev,
+    }
     config["tool_info"] = {
         "program": "CrispyCookie",
         "version": __version__,
     }
-    with open(output_folder / ".crispycookie.json", "w") as fp:
+    with open(project_dir / ".crispycookie.json", "w") as fp:
         json.dump(config, fp, indent=4)
 
 
+def get_crispycookie_source(p):
+    if hasattr(p, "read"):
+        config = json.load(p)
+    else:
+        with open(p) as fp:
+            config = json.load(fp)
+    try:
+        s = config["source"]
+        return s["repo"], s["rev"]
+    except KeyError:
+        return None
+
+
+def do_update(args):
+    from .rebase import upgrade_project
+    project_dir = Path(args.project)
+    project_config = project_dir / ".crispycookie.json"
+    cli_config = args.config if args.config else None
+
+    if not args.branch:
+        print(f"Missing template-only branch name.", file=sys.stderr)
+        return 1
+
+    print(f"Project file:  {project_config}")
+    if not project_dir.is_dir():
+        print(f"Invalid project directory {project_dir}", file=sys.stderr)
+        return 1
+
+    project_source = cli_source = None
+    if cli_config:
+        cli_source = get_crispycookie_source(cli_config)
+
+    if project_config.is_file():
+        project_source = get_crispycookie_source(project_config)
+    else:
+        if not cli_source:
+            print(f"Missing project configuration file: {project_config}  "
+                  "Use '--config' to use an alternate crispycookie.json file.")
+            return 3
+
+    # XXX: Check to see if branch exists?
+
+    if cli_source and project_source:
+        print(f"Overridding repo details:  {project_source} with {cli_source}")
+        source = cli_source
+        config_file = cli_config.name
+    elif cli_source:
+        print(f"Using CLI values to bootstrap project:  {cli_source}")
+        source = cli_source
+        config_file = cli_config.name
+    elif project_source:
+        print(f"Using project defaults:  {project_source}")
+        source = project_source
+        config_file = project_config
+    else:
+        raise AssertionError("This shouldn't happen")
+
+    if cli_config:
+        config_file = cli_config.name
+        print(f"Using external config file {config_file} set on command line")
+
+    repo, checkout = source
+
+    if args.repo:
+        repo = args.repo
+    if args.checkout:
+        checkout = args.checkout
+    tc = get_local_repo(repo, checkout)
+    upgrade_project(tc, project_dir, args.branch, config_file, do_build, remote_ops=args.remote)
+
+
 def _copy_tree(src: Path, dest: Path, layer_info=None):
     if not dest.is_dir():
         dest.mkdir()
@@ -358,6 +474,29 @@ def _copy_tree(src: Path, dest: Path, layer_info=None):
             raise ValueError(f"Unsupported file type {p}")
 
 
+def get_local_repo(repo: str, checkout: str):
+    abbreviations = {}
+
+    local_clone_dir = "~/.crispy_cookie/repos"
+
+    # Try local directory first.  (is_dir() may fail with git url)
+    template_dir = None
+    try:
+        if Path(repo).expanduser().is_dir():
+            template_dir = repo
+    except OSError:
+        pass
+
+    if not template_dir:
+        # Assume remote repository
+        template_dir = clone(repo, checkout, local_clone_dir, True)
+
+    tc = TemplateCollection(Path(template_dir))
+    tc.repo = repo
+    tc.rev = checkout
+    return tc
+
+
 def main():
     def add_repo_args(parser):
         parser.add_argument("repo", help="Path to local or remote repository "
@@ -365,12 +504,13 @@ def main():
         parser.add_argument("-c", "--checkout", help="Branch, tag, or commit "
                             "to checkout from git repository.")
     parser = ArgumentParser()
-    parser.set_defaults(function=None)
+    parser.set_defaults(function=None, do_repo_prep=True)
     parser.add_argument('--version', action='version',
                         version='%(prog)s {version}'.format(version=__version__))
 
     subparsers = parser.add_subparsers()
 
+    # COMMAND:  crispy_cookie config
     config_parser = subparsers.add_parser(
         "config",
         description="Make a fresh configuration based on named template layers")
@@ -386,18 +526,21 @@ def main():
     config_parser.add_argument("-o", "--output", type=FileType("w"),
                                default=sys.stdout)
 
+    # COMMAND:  crispy_cookie list
     list_parser = subparsers.add_parser("list",
                                         description="List available template layers")
     list_parser.set_defaults(function=do_list)
     add_repo_args(list_parser)
 
+    # COMMAND:  crispy_cookie build
     build_parser = subparsers.add_parser("build",
                                          description="Build from a config file")
     build_parser.set_defaults(function=do_build)
     add_repo_args(build_parser)
     build_parser.add_argument("--config", type=FileType("r"),
-                              help="JSON config file.  Needed the first time "
-                              "a project is built.")
+                              help="JSON config file.  "
+                              "After the project the configuration is saved to "
+                              "'.crispycookie.json' in the project folder.")
     build_parser.add_argument("-o", "--output",
                               default=".", metavar="DIR",
                               help="Top-level output directory.  Or the project "
@@ -406,29 +549,37 @@ def main():
     build_parser.add_argument("--verbose", action="store_true", default=False,
                               help="More output for var handling and such")
 
+    # COMMAND:  crispy_cookie update
+    update_parser = subparsers.add_parser(
+        "update",
+        description="Reapply a template with an updated version or configuration")
+    update_parser.set_defaults(function=do_update, do_repo_prep=False)
+    update_parser.add_argument("project", metavar="DIR",
+                               help="Project to be updated.")
+    update_parser.add_argument("--repo", help="Updated path to templates.  By "
+                               "default the same repo will be reused.")
+    update_parser.add_argument("-c", "--checkout", help="Branch, tag, or commit "
+                               "to checkout from git repository.")
+    update_parser.add_argument("--branch", default="cookiecutter",
+                               help="Template-only branch. "
+                               "Defaults to %(default)s")
+    update_parser.add_argument("--config", type=FileType("r"),
+                               help="Explicitly set JSON config file.")
+    update_parser.add_argument("--verbose", action="store_true", default=False,
+                               help="More output for var handling and such")
+    update_parser.add_argument("--no-remote", dest="remote", action="store_false", default=True,
+                               help="Disable remote git operations.")
+
     args = parser.parse_args()
     if args.function is None:
         sys.stderr.write(parser.format_usage())
         sys.exit(1)
 
-    abbreviations = {}
-
-    local_clone_dir = "~/.crispy_cookie/repos"
-
-    # Try local directory first.  (is_dir() may fail with git url)
-    template_dir = None
-    try:
-        if Path(args.repo).expanduser().is_dir():
-            template_dir = args.repo
-    except OSError:
-        pass
-
-    if not template_dir:
-        # Assume remote repository
-        template_dir = clone(args.repo, args.checkout, local_clone_dir, True)
-
-    tc = TemplateCollection(Path(template_dir))
-    return args.function(tc, args)
+    if args.do_repo_prep:
+        tc = get_local_repo(args.repo, args.checkout)
+        return args.function(tc, args)
+    else:
+        return args.function(args)
 
 
 if __name__ == "__main__":
